﻿// Created by Eric Wong
// http://videoinpicture.wikidot.com
// August 26th, 2008

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;


namespace WindowsFormsApplication1
{
    public partial class Form1 : Form
    {
        public static String DirectoryToRestructure;

        public const String MSFExtension = @".msf";

        // Not a real extension since this is for a folder
        public const String SBDExtension = @".sbd";

        public const String OLDExtension = @".old";

        public Form1()
        {
            InitializeComponent();
            Finished.Text = "Ready";
            Finished.Visible = true;
        }

        private void BrowseforTarget_Click(object sender, EventArgs e)
        {
            folderBrowserDialog1.Description = @"Browse for Folder to Restructure";
            if (folderBrowserDialog1.ShowDialog() == DialogResult.OK)
            {
                sourcePath = folderBrowserDialog1.SelectedPath;

                if (sourcePath != "")
                    SourceDirectoryTextBox.Text = sourcePath;
            }
        }

        private void ClearFields_Click(object sender, EventArgs e)
        {
            Finished.Text = "";
            richTextBox1.Clear();
            progressBar1.Value = 0;
        }

        private void EnableDisableButtons(bool EnableDisable)
        {
            if (EnableDisable == false)
            {
                Stop = false;
                SourceDirectoryTextBox.Enabled = false;
                ClearFields.Enabled = false;
                RestructureButton.Enabled = false;
                BrowseforTarget.Enabled = false;
            }
            else
            {
                Stop = true;
                Application.DoEvents();
                SourceDirectoryTextBox.Enabled = true;
                ClearFields.Enabled = true;
                RestructureButton.Enabled = true;
                BrowseforTarget.Enabled = true;
                Finished.Text = "Stopped";
                richTextBox1.AppendText("Stopped.\r");
            }
        }

        string sourcePath = "";
        string targetPath = "";
        int TotalItemsCopied = 0;
        int TotalItems = 0;
        bool Stop = true;

        private void RestructureButton_Click(object sender, EventArgs e)
        {
            richTextBox1.Clear();
            Finished.Text = "";
            Finished.Visible = true;
            progressBar1.Value = 0;
            TotalItems = 0;
            TotalItemsCopied = 0;    
            EnableDisableButtons(false);

            // Check to see if the source directory exists
            sourcePath = SourceDirectoryTextBox.Text.Trim().ToString();

            if (sourcePath.Trim() == "" || sourcePath.Trim().Length < 1 && !System.IO.Directory.Exists(sourcePath))
            {
                Finished.Text = "Source directory does not exist / Select a source directory first";
                EnableDisableButtons(true);
                return;
            }
            else
            {
                richTextBox1.AppendText("Errors will be shown here.\r");
                richTextBox1.AppendText("------------------------------------------------------------------\r");
            }


            // To copy a folder's contents to a new location:
            // Create a new target folder, if necessary.
            if (sourcePath[sourcePath.Length-1] != Path.DirectorySeparatorChar)
            {
                targetPath = sourcePath.Insert(sourcePath.Length, "2");
            }
            else
            {
                targetPath = sourcePath.Insert(sourcePath.Length - 1, "2");
            }


            if (System.IO.Directory.Exists(sourcePath))
            {
                TotalItemsCopied = 0;
                countFiles(sourcePath);

                // Create the new directory
                if (targetPath[targetPath.Length - 1] != Path.DirectorySeparatorChar)
                    targetPath += Path.DirectorySeparatorChar;
                if (!Directory.Exists(targetPath))
                {
                    try
                    {
                        Directory.CreateDirectory(targetPath);
                    }
                    catch (Exception)
                    {
                        Finished.Text = "Invalid Directory";
                        EnableDisableButtons(true);
                        return;
                    }
                }
                else
                {
                    try
                    {
                        Directory.Delete(targetPath, true);
                        Directory.CreateDirectory(targetPath);
                    }
                    catch (Exception)
                    {
                        Finished.Text = "Invalid Directory / Directory is in use";
                        EnableDisableButtons(true);
                        return;
                    }
                }

                // Make a copy of the source directory in the destination directory
                copyDirectory(sourcePath, targetPath);

                string[] allFilesAndFolders = Directory.GetFileSystemEntries(targetPath);

                // Restructure the MSF/SBD files/folders in the root of the targetPath directory (non-subfolders)
                string[] files = System.IO.Directory.GetFiles(targetPath);      // Get actual files
                string[] directoriesSameLevel = System.IO.Directory.GetDirectories(targetPath);     // Get the directories in seen in the current targetPath
                string fileName = "";
                string fileExtension = "";
                bool msfFile_sbdFolder_bothExist = false;

                if (files != null && directoriesSameLevel != null)
                {
                    foreach (string filePath in files)
                    {
                        if (Stop == true)
                        {
                            EnableDisableButtons(true);
                            return;
                        }
                        Application.DoEvents();
                        // Use static Path methods to extract only the file name from the path.
                        fileName = System.IO.Path.GetFileName(filePath);
                        fileExtension = System.IO.Path.GetExtension(filePath);
                        Finished.Text = filePath;
                        Finished.Visible = true;
                        msfFile_sbdFolder_bothExist = false;

                        //richTextBox1.AppendText("fileExtension: " + fileExtension + "\r");

                        // See if there is a corresponding .sbd folder for this .msf file
                        if (fileExtension.Length > 0 && fileExtension.ToLower() == MSFExtension.ToLower())
                        {
                            // Look at the directories in the current level and see if there is a corresponding .sbd folder
                            foreach (string tempDirectoryPath in directoriesSameLevel)
                            {
                                if (tempDirectoryPath.ToLower().EndsWith(SBDExtension.ToLower()))
                                {
                                    string tempDirectoryName = tempDirectoryPath.Substring(0, tempDirectoryPath.Length - SBDExtension.Length);
                                    string tempFileName = filePath.Substring(0, filePath.Length - MSFExtension.Length);

                                    //richTextBox1.AppendText("tempDirectoryName: " + tempDirectoryName + "    " + tempFileName + "\r");

                                    if (tempDirectoryName.Length > 0 && tempFileName.ToLower() == tempDirectoryName.ToLower())
                                    {
                                        msfFile_sbdFolder_bothExist = true;
                                        break;
                                    }
                                }
                            }

                            if (msfFile_sbdFolder_bothExist)
                            {
                                try
                                {
                                    // This renames the file with the OLDExtension added
                                    System.IO.File.Move(filePath, filePath + OLDExtension);
                                }
                                catch (Exception)
                                {
                                    richTextBox1.AppendText("Error 1: " + fileName + "---- Could not rename." + "\r");
                                }
                            }
                        }

                        TotalItemsCopied++;
                        UpdateProgress((int)((double)TotalItemsCopied / (double)TotalItems * 100.0));
                    }
                }

                // Look in each subfolder of the targetPath and restructure the .msf files if a corresponding .sbd folder is found
                foreach (string Element in allFilesAndFolders)
                {
                    if (Stop == true)
                    {
                        EnableDisableButtons(true);
                        return;
                    }
                    Application.DoEvents();

                    // The Element is a folder
                    if (Directory.Exists(Element))
                    {
                        files = System.IO.Directory.GetFiles(Element);      // Get actual files
                        directoriesSameLevel = System.IO.Directory.GetDirectories(Element);     // Get the directories in seen in the current targetPath
                        fileName = "";
                        fileExtension = "";
                        msfFile_sbdFolder_bothExist = false;

                        if (files != null && directoriesSameLevel != null)
                        {
                            foreach (string filePath in files)
                            {
                                if (Stop == true)
                                {
                                    EnableDisableButtons(true);
                                    return;
                                }
                                Application.DoEvents();

                                // Use static Path methods to extract only the file name from the path.
                                fileName = System.IO.Path.GetFileName(filePath);
                                fileExtension = System.IO.Path.GetExtension(filePath);
                                Finished.Text = filePath;
                                Finished.Visible = true;
                                msfFile_sbdFolder_bothExist = false;

                                // See if there is a corresponding .sbd folder for this .msf file
                                if (fileExtension.Length > 0 && fileExtension.ToLower() == MSFExtension.ToLower())
                                {
                                    // Look at the directories in the current level and see if there is a corresponding .sbd folder
                                    foreach (string tempDirectoryPath in directoriesSameLevel)
                                    {
                                        if (tempDirectoryPath.ToLower().EndsWith(SBDExtension.ToLower()))
                                        {
                                            string tempDirectoryName = tempDirectoryPath.Substring(0, tempDirectoryPath.Length - SBDExtension.Length);
                                            string tempFileName = filePath.Substring(0, filePath.Length - MSFExtension.Length);

                                            //richTextBox1.AppendText("tempDirectoryName: " + tempDirectoryName + "    " + tempFileName + "\r");

                                            if (tempDirectoryName.Length > 0 && tempFileName.ToLower() == tempDirectoryName.ToLower())
                                            {
                                                msfFile_sbdFolder_bothExist = true;
                                                break;
                                            }
                                        }
                                    }

                                    if (msfFile_sbdFolder_bothExist)
                                    {
                                        try
                                        {
                                            // This renames the file with the OLDExtension added
                                            System.IO.File.Move(filePath, filePath + OLDExtension);
                                        }
                                        catch (Exception)
                                        {
                                            richTextBox1.AppendText("Error 2: " + fileName + "---- Could not rename." + "\r");
                                        }
                                    }
                                }

                                TotalItemsCopied++;
                                UpdateProgress((int)((double)TotalItemsCopied / (double)TotalItems * 100.0));
                            }
                        }
                    }
                }
            }

            Finished.Text = "Finished! New Directory: " + targetPath;
            richTextBox1.AppendText("------------------------------------------------------------------\r");
            richTextBox1.AppendText("Finished! New Directory: " + targetPath + "\r");
            EnableDisableButtons(true);
        }

        // Count the number of main subdirectories and files
        // This is not the whole total
        public void countFiles(string Src)
        {
            String[] Files;

            Files = Directory.GetFiles(Src);
            foreach (string Element in Files)
            {
                Application.DoEvents();
                TotalItems++;
            }

            Files = Directory.GetDirectories(Src);
            foreach (string Element in Files)
            {
                Application.DoEvents();
                TotalItems++;
            }
        }

        // Function to copy directory structure recursively
        public void copyDirectory(string Src, string Dst)
        {
            Application.DoEvents();
            String[] Files;

            if (Dst[Dst.Length - 1] != Path.DirectorySeparatorChar) Dst += Path.DirectorySeparatorChar;
            if (!Directory.Exists(Dst))
            {
                Directory.CreateDirectory(Dst);
            }
            Files = Directory.GetFileSystemEntries(Src);
            foreach (string Element in Files)
            {
                if (Stop == true)
                {
                    EnableDisableButtons(true);
                    return;
                }
                Application.DoEvents();
                Finished.Text = Element;
                // Sub directories
                if (Directory.Exists(Element))
                {
                    copyDirectory(Element, Dst + Path.GetFileName(Element));
                }
                // Files in directory
                else
                {
                    try
                    {
                        File.Copy(Element, Dst + Path.GetFileName(Element), true);
                    }
                    catch
                    {
                        richTextBox1.AppendText("Error 2: " + Element + "\r");
                    }
                }
            }
        }

        public void UpdateProgress(int CurrentProgress)
        {
            if (CurrentProgress <= 100 && CurrentProgress >= 0)
            {
                progressBar1.Value = CurrentProgress;
            }
        }

        private void StopButton_Click(object sender, EventArgs e)
        {
            Stop = true;
        }

    }
}
